
# This procedure takes one arg:  extract potential of the grain
# (decimal).  For example, 2-Row (per Briess) is 79.5%, so we'd
# use .795.  #  46.31 (%yield) = points/lb/gal
# Source:
#    "Understanding Specific Gravity and Extract" by Martin P. Manning
#          Republished from BrewingTechniques' September/October 1993.
#

proc extpot_to_ppg {extract_potential} {
   if {$extract_potential > 1} {
      set extract_potential [expr {$extract_potential/100.00}]
   }
   set ppg [expr {46.31 * $extract_potential}]
}

proc SG_to_Plato {SG} { return [format %.02f [expr {1000*($SG - 1)/4}]] }

# Calculate points/pounds/gallon (ppg)

proc get_ppg {sg gals lbs} {
   set ppg [expr {round(((($sg-1)*1000) * $gals)/$lbs)}]
   return $ppg
}

proc grain_valid {idx} {
   global Grain

   if {[string length $Grain($idx:name)] > 0} {
      if {![streqx $Grain($idx:lb) "" 0] || ![streqx $Grain($idx:oz) "" 0]} {
         return 1
      }
   }
   set Grain($idx:percent) ""
   return 0
}


# gets OGSG and total weight of grain
proc get_calculated_OG {} {
   global Grain OGPL OGSG totalpts batchsize totalweight grains
   global masheff

   foreach idx $Grain() { set totalpts($idx) 0 }
   set totalpts() 0
   set totalweight 0

   foreach idx $Grain() {
      # clear a couple of variables
      set Grain($idx:percent) 0
      set lbs 0
      if {[grain_valid $idx]} {
         if {![streqx $Grain($idx:lb) "" 0]} { set lbs $Grain($idx:lb) }
         if {![streqx $Grain($idx:oz) "" 0]} {
            set lbs [expr {1.00*$lbs+($Grain($idx:oz)/16.000)}]
         }
         set totalweight [expr {1.00*$totalweight+$lbs}]
         set name $Grain($idx:name)
         set ppg [lindex $grains($name) 0]
         set totalpts($idx) [expr {1.00*$lbs*$ppg}]
         set totalpts() [expr {1.00*($totalpts($idx) + $totalpts())}]
         addgrain $idx $Grain($idx:name) $lbs
      } else {
         set totalpts($idx) 0
      }
   set OGSG [expr {1.00 + ($masheff*$totalpts()/$batchsize)/1000}]
   }
}


proc calc_grain {} {
   global Grain totalweight OGPL OGSG SRM dSRM grainbill partialwt
   global totalpts batchsize

   set OGPL 0 ; set OGSG 1 ; set SRM 0 ; set dSRM 0 ; set grainbill [list]

   get_calculated_OG
   foreach idx $Grain() {
      set lbs 0
      if {[grain_valid $idx]} {
         if {![streqx $Grain($idx:lb) "" 0]} { set lbs $Grain($idx:lb) }
         if {![streqx $Grain($idx:oz) "" 0]} {
            set lbs [expr {1.00*$lbs+($Grain($idx:oz)/16.000)}]
         }
      }

      if {[streqx $lbs "" 0] == 0} {
         set tmp [expr {round(10000.00*(1.00*$lbs/$totalweight))/100.0}]
         set Grain($idx:fpercent) $tmp
         set Grain($idx:percent) [format %.2f $tmp]
      }
   }
   calc_si_stepmash
}

proc masheff {sg vol lbs} {
   global actualppg totalpts totalweight
   set tp [expr {1.00*$totalpts()/$totalweight}]
   return [format %.01f [expr {100.00*$actualppg/$tp}]]
}


# Calculate the specific gravity contribution for individual grains from
# the grain's gravity, weight (pounds), and batch size (gallons):

proc partialgrav {ppg weight batchsize} {
   global masheff maxppg

   if {$batchsize > 0} {
      set partial [expr {(($weight*$ppg)/$batchsize)}]
      set maxppg [expr {1.000*$maxppg + $partial}]
      set partial [expr {(($masheff*$partial)/1000.000)}]
   } else {
      set partial 0
   }
   return $partial
}

proc partialweight {ppg partialgrav batchsize meff} {

   set ppg [expr {1.00 * $ppg/1000.000}]
   set pweight [expr {1.000 * ($partialgrav * $batchsize) / ($meff * $ppg)}]
   return $pweight
}



proc addgrain {idx grain lb} {
   global grains grainbill OGPL OGSG ogpltext SRM batchsize sgpartial
   global srmpartial dSRM srmchart ogtarget srmtarget ogtargpl Grain

   set ppg [lindex $grains($grain) 0]
   set color  [lindex $grains($grain) 1]
   set sgpartial [partialgrav $ppg $lb $batchsize]
   set Grain($idx:sgpartial) $sgpartial
   set srmpartial [partialcolor $color $lb $batchsize]
   set OGSG [format %.03f [expr {1 + (($OGSG-1.000) + $sgpartial)}]]
   set ogtarget $OGSG
   set OGPL [SG_to_Plato $OGSG]
   set ogtargpl "$OGPL P"
   set SRM [expr {$SRM + $srmpartial}]
   set dSRM [expr {int($SRM)}]
   set srmtarget $dSRM

   if {$dSRM > 44} {
      .c.srmimg configure -background $srmchart(44)
   } else {
      .c.srmimg configure -background $srmchart($dSRM)
   }
   lappend grainbill [list $grain $lb]
   set ogpltext "([SG_to_Plato $OGSG] P)"
   check_bjcp
   return [llength $grainbill]
}


